<?php
/*
+-------------------------------------------------------------------------+
| @FILE_NAME:   class.Jabber.inc.php                                      |
| @CLASS_NAME:  Jabber                                                    |
|                                                                         |
| @AUTHOR:      Alexandre Correia    - alexandrecorreia@celepar.pr.gov.br |
|               [NUTS] Rodrigo Souza - rodsouza@celepar.pr.gov.br         |
|                                                                         |
| @DATE:        Seg 18 Dez 2006 10:10:59 BRST                             |
| @LAST_CHANGE: Qui 21 Dez 2006 09:10:28 BRST::                           |
|                                                                         |
| @BRIEF:       Classe para acesso e comunicao ao                       |
|               Servidor Jabberd2 com TLS                                 |
+-------------------------------------------------------------------------+
*/

require_once "class.Socket.inc.php";
require_once "class.XML.inc.php";

class Jabber extends Socket
{
	private $charset;

	private $server;
	private $port;
						
	private $username;
	private $password;

	private $jid;
	private $resource;

	private $log_error = false;
	
	function __construct($pJid = false, $pPassword = false, $pPort = false)
	{
      if ( $pJid && $pPassword )
      {
         $this->xml = new XML();
         $this->charset = 'UTF-8';
         
         $this->jid        = substr($pJid, 0, strpos($pJid, '/'));

         $this->username   = substr($pJid, 0, strpos($pJid, '@'));
         $this->password   = $pPassword;

         $pJid = substr($pJid, strpos($pJid, '@')+1);
         $this->resource   = substr($pJid, strpos($pJid, '/')+1);

         $this->server     = substr($pJid, 0, strpos($pJid, '/'));;
         $this->port       = $pPort ? $pPort : 5222;

         if ( $this->open('tcp://'.$this->server.':'.$this->port, 1, 1) )
            $this->setConnect();
      }
      else
      {
         // LOG ERRO
      }
	}

	function __destruct()
	{
	}

   /*
    * funcoes derivadas de outras classes
    */
   protected final function xmlize($pXML)
   {
      return $this->xml->xmlize($pXML);
   }
   
	protected final function sendPacket($pPacket)
	{
		return $this->write($pPacket);
	}

   /*
    * funcoes implementadas nesta classe
    */

   /*
    * funcoes privadas
    */
	private final function _iq($pType = false, $pId = false, $pTo = false, $pFrom = false, $pXmlns = false, $pLoad = false )
	{
		$xml  = "<iq type='" . $pType . "' id='" . $pId . "'";
		$xml .= ( $pTo ) ? " to='" . $pTo . "'" : "";
		$xml .= ( $pFrom ) ? " from='" . $pFrom . "'" : "";

		if ( $pXmlns == "vcard-temp" )
      {
			$xml .= ">";
			$xml .= $pLoad;
		}
      else
      {
			$xml .= "><query xmlns='" . $pXmlns . "'";
			$xml .= ( $pLoad ) ? ">" . $pLoad . "</query>" : "/>";
		}

		$xml .= "</iq>";
		
		return $xml;	
	}

   private final function authPlain()
   {
		$xml  = "<username>" . $this->username . "</username>";
		$xml .= "<password>" . $this->password . "</password>";
		$xml .= "<resource>" . $this->resource . "</resource>";

		if ( !$this->sendIq('set', 'auth_1', NULL, NULL, "jabber:iq:auth", $xml) )
      	{
		   return false;
		}
      	usleep(90000);
   }

   private function setConnect()
   {
      $xml  = "<?xml version='1.0' encoding='UTF-8'?><stream:stream to='" . $this->server . "' xmlns='jabber:client' xmlns:stream='http://etherx.jabber.org/streams' version='1.0'>";
      $this->write($xml);
      $xml = $this->read();

      $xml = $this->xmlize($xml);

      if ( @array_key_exists('starttls', $xml['stream:stream']['#']['stream:features']['0']['#']) &&
           @array_key_exists('required', $xml['stream:stream']['#']['stream:features']['0']['#']['starttls']['0']['#']) )
         $this->setTLS();
      else
         $this->simple_authentication();		
   }

   private function setTLS()
   {
      $xml = "<starttls xmlns='urn:ietf:params:xml:ns:xmpp-tls'/>";
      $this->write($xml);

      $xml = $this->read();
      $xml = $this->xmlize($xml);

      if ( array_key_exists('proceed', $xml) )
      {
		   stream_socket_enable_crypto($this->returnSocket(), TRUE, STREAM_CRYPTO_METHOD_SSLv23_CLIENT);

         $xml = "<stream:stream to='" . $this->server . "' xmlns='jabber:client' xmlns:stream='http://etherx.jabber.org/streams' version='1.0'>";
         $this->write($xml);

         $xml = $this->read();
         $xml = $this->xmlize($xml);

         socket_set_blocking($this->returnSocket(), 0);

         $this->authPlain();
      }
      else
      {
         // LOG ERRO
      }
   }

   private function simple_authentication()
   {
         socket_set_blocking($this->returnSocket(), 0);
         
         $this->authPlain();
   }

   /*
    * funcoes do pacote
    */

	protected final function presence($pType = false, $pTo = false, $pShow = false, $pStatus = false, $pPriority = false)
	{
		$xml  = "<presence";
		$xml .= ($pTo)   ? " to='{$pTo}'" : '';
		$xml .= ($pType) ? " type='{$pType}'" : '';
		$xml .= ($pStatus || $pShow || $pPriority) ? ">" : "/>";

		$xml .= ($pStatus)   ? "	<status>{$pStatus}</status>" : '';
		$xml .= ($pShow)     ? "	<show>{$pShow}</show>" : '';
		$xml .= ($pPriority) ? "	<priority>{$pPriority}</priority>" : '';
		$xml .= ($pStatus || $pShow || $pPriority) ? "</presence>" : '';

		if ( $this->sendPacket($xml) )
		{
	        usleep(90000);

			return true;
		}
		else
		{
			$this->writeLog("ERROR: send_presence() #1");
			return false;
		}
	}

	protected final function readSocketFromServer()
	{
	  $retorno = "<resposta>" . $this->read(true) . "</resposta>";
      $this->array_teste[] = $retorno;
      $retorno = $this->xmlize($retorno);
      $retorno = $retorno['resposta']['#'];
      return $retorno;
	}

	protected final function sendIq($pType = false, $pId = false, $pTo = false, $pFrom = false, $pXmlns = false, $pLoad = false )
	{
		if ( !preg_match("/^(get|set|result|error)$/i", $pType, $matches, PREG_OFFSET_CAPTURE) )
		{
			$this->writeLog("ERROR: _iq() #2 - type must be 'get', 'set', 'result' or 'error'");
			return false;
		}

		if ( $this->sendPacket($this->_iq($pType, $pId, $pTo, $pFrom, $pXmlns, $pLoad)) )
        {
			return true;
		}
		return false;
	}

   protected final function writeLog($pLog = false)
   {
      switch ( substr($pLog, 0, strpos($pLog, ':')) )
      {
         case 'ERROR' :
                        if ( $this->log_error === true )
                        {
                           $log = NULL;
                           $log .= $pLog . ' :: ';
                           $log .= $this->jid . ' :: ';
                           $log .= date('m/d/Y H:i:s');
                           $log .= "\n";
                           if ( $fp = fopen ($this->log_error_file, "a+") )
                           {
                              fwrite($fp, $log);
                              fclose($fp);
                           }
                        }
         // end case 'ERROR'
         break;
      }
   }

}
?>
